@@ -17,7 +17,8 @@ from fluidtopics.markdown import __version__ as md2ft_version
 from fluidtopics.markdown.errors import MD2FTError
 from fluidtopics.markdown.ftmap import FTMap
 from fluidtopics.markdown.ftsession import FTSession
-from fluidtopics.markdown.metadata import FT_ORIGIN_ID, FT_LANG, get_md_metas, MISSING_TITLE
+from fluidtopics.markdown.metadata import FT_ORIGIN_ID, FT_LANG, get_md_metas, MISSING_TITLE, MD2FT_MEDIA_DIR, MD2FT_AUDIENCE, MD2FT_ATTACHMENT_DIR
+from fluidtopics.markdown.attachment import DEFAULT_ATTACHMENT_DIR, DEFAULT_MEDIA_DIR, collect_attachments
 
 logger = logging.getLogger("fluidtopics.markdown")
 click_log.basic_config(logger)
@@ -74,7 +75,7 @@ def dir_content(
                 logger.debug(f"Found directory description in {dirdesc}")
                 skip.append(dirdesc.relative_to(relto))
                 if meta_include_list:
-                    audience = dirmetas.get("audience", None)
+                    audience = dirmetas.get(MD2FT_AUDIENCE, None)
                     if audience is None:
                         logger.warning(
                             f"Skipping {file.relative_to(relto).as_posix()} no audience given as meta while include_list = {meta_include_list}"
@@ -146,7 +147,7 @@ def dir_content(
         elif file.is_file() and file.match("*.md"):
             metas, topic_content, heading_title = get_md_metas(file, implicit_meta, str(file.relative_to(relto)) == 'README.md',
                                                    dev_mode=dev_mode)
-            audience = metas.get("audience", None)
+            audience = metas.get(MD2FT_AUDIENCE, None)
             if meta_include_list:
                 if audience is None:
                     logger.warning(
@@ -169,10 +170,11 @@ def dir_content(
     return content
 
 
-def _create_zip_file(output: str, pinput: Path, image_dir: str, ftmap_file: TemporaryFile, collected_files: List[Path],
+def _create_zip_file(output: str, pinput: Path, image_dir: str, ftmap: FTMap, collected_files: List[Path],
                      dev_mode: bool):
+    attached_files = [attachment["filePath"] for attachment in ftmap.attachments]
+    logger.debug(f"Attached files: {attached_files}")
     with ZipFile(output, mode="w", compression=ZIP_DEFLATED) as archive:
-        archive.writestr("generated.ftmap", data=ftmap_file.read())
         # NB: we need to use glob.glob and not pathlib.Path.rglob
         #     because of: https://bugs.python.org/issue33428
         #     see also: https://stackoverflow.com/questions/46529760/getting-glob-to-follow-symlinks-in-python
@@ -180,7 +182,11 @@ def _create_zip_file(output: str, pinput: Path, image_dir: str, ftmap_file: Temp
             f = Path(fn)
             if f.match("*.ftmap"):
                 continue
-            if f.is_dir() or ((pinput / Path(image_dir)) in f.parents):
+            if (
+                f.is_dir()
+                or ((pinput / Path(image_dir)) in f.parents)
+                or (f.relative_to(pinput).as_posix() in attached_files)
+            ):
                 archive.write(f, arcname=f.relative_to(pinput))
             elif f.is_file() and f.suffix.lower() in [
                 ".png",
@@ -194,10 +200,16 @@ def _create_zip_file(output: str, pinput: Path, image_dir: str, ftmap_file: Temp
             elif f.relative_to(pinput) in collected_files:
                 # get MD content with stripped-out metadata
                 _, md_content, _ = get_md_metas(f, dev_mode=dev_mode)
+                if md_content == '':
+                    logger.info(f'{f.relative_to(pinput)} has no content, it will be ignored')
+                    ftmap.remove_node(f.relative_to(pinput).name)
+                    continue
                 archive.writestr(str(f.relative_to(pinput)), md_content)
             else:
                 logger.debug(f"{f.relative_to(pinput)} not collected in archive.")
 
+        archive.writestr("generated.ftmap", data=ftmap.to_string())
+
 
 def do_collect(
         input: str,
@@ -206,12 +218,12 @@ def do_collect(
         implicit_meta: bool,
         dev_mode: bool = False,
 ):
-    """Collect markdown files and create FTML archive with a book FT Map
+    """Collect markdown files and create FT Archive with a book FT Map
     built from the metadata found in the markdown files (unless implicit_meta is True)
     The created archive may then be uploaded to FT portal using `upload`.
     """
-    pinput = Path(input)
-    root_readme = list(pinput.glob("README.md"))
+    input_path = Path(input)
+    root_readme = list(input_path.glob("README.md"))
     metas = None
     root_readme_title = MISSING_TITLE
     # We always collect metas from root README is there is some
@@ -222,11 +234,11 @@ def do_collect(
         logger.debug("Implicit meta MODE")
         implicit_metas_values = {
             # if implicit meta is set to True, build the title from pinput dir name
-            "media_dir": "images",
-            SemanticMetadata.TITLE: f"{pinput.name}",
+            MD2FT_MEDIA_DIR: DEFAULT_MEDIA_DIR,
+            SemanticMetadata.TITLE: f"{input_path.name}",
             FT_ORIGIN_ID: input,
             "Category": "Technical Notes",
-            "audience": "internal",
+            MD2FT_AUDIENCE: "internal",
         }
         # Replace provided metas value if any
         if metas:
@@ -234,21 +246,26 @@ def do_collect(
         metas = implicit_metas_values
 
     if metas is None and not implicit_meta:
-        raise MD2FTError(f"No README.md found in {pinput}")
+        raise MD2FTError(f"No README.md found in {input_path}")
 
     logger.debug(f"root metas = {metas}")
     # Get media_dir value or use default one (images/)
-    image_dir = metas.get("media_dir", "images")
-
+    image_dir = metas.get(MD2FT_MEDIA_DIR, DEFAULT_MEDIA_DIR)
     skipped = [Path(image_dir)]
+
+    attachment_dir = Path(metas.get("attachment_dir", DEFAULT_ATTACHMENT_DIR))
+    map_attachments = collect_attachments(input_path, attachment_dir)
+    if map_attachments:
+        skipped.append(attachment_dir)
+
     # skip root README iff there is NO content in it
     # and we are in not implicit mode
     if not implicit_meta and not root_content:
-        skipped.append(root_readme.relative_to(pinput))
+        skipped.append(root_readme.relative_to(input_path))
     # Recursively collect md files beginning at pinput
     collected = dir_content(
-        pinput,
-        pinput,
+        input_path,
+        input_path,
         skipped,
         meta_include_list=included_audiences,
         implicit_meta=implicit_meta,
@@ -277,17 +294,14 @@ def do_collect(
         ft_metas = metas
     logger.debug(f"FT metas = {ft_metas}")
     ftmap.add_metas(ftmap.root, ft_metas)
+    ftmap.add_attachments(map_attachments)
     # Build FTMAP ToC with collected items + return files list to be zipped
     collected_files = ftmap.populate_toc(ftmap.get_toptoc(), None, collected.values())
     if len(collected_files) == 0:
         raise MD2FTError(f"0 collected file, check your input ({input}) directory and/or --include rules.")
     logger.info(f"Collected {len(collected_files)} markdown files in FTMap.")
     logger.debug(f"FTMap collected_files= {collected_files}")
-    # Create the output zip file
-    with TemporaryFile() as ftmap_file:
-        ftmap.write(ftmap_file)
-        ftmap_file.seek(0)
-        _create_zip_file(output, pinput, image_dir, ftmap_file, collected_files, dev_mode=dev_mode)
+    _create_zip_file(output, input_path, image_dir, ftmap, collected_files, dev_mode=dev_mode)
 
 
 def do_publish(
@@ -296,25 +310,30 @@ def do_publish(
         source_id: str,
         dry_run: bool,
 ):
-    """Upload and publish an FTML archive to an FT portal."""
-    if not dry_run:
-        # Verify that the server is reachable and has FTML source exists
-        ft_session.get("/api/admin/khub/sources", json_result=False)
+    """Upload and publish an FT Archive to an FT portal."""
 
-    # Upload FTML zip archive
     if type(zip) == str:
         zipfile = {"file": ("md2ft_" + Path(zip).name, Path(zip).open("rb"))}
     else:
         zipfile = {"file": ("md2ft_" + Path(zip.name).name, zip)}
 
-    if not dry_run:
-        answer = ft_session.post(f"/api/admin/khub/sources/{source_id}/upload", files=zipfile)
+    if dry_run:
+        logger.info(f"DRY RUN: {zipfile['file'][0]} file NOT published.")
+        return
+
+    answer = ft_session.post(f"/api/admin/khub/sources/{source_id}/upload",
+                             files=zipfile,
+                             json_result=False,
+                             check_status=False)
+    if answer.ok:
         logger.info(f"{zipfile['file'][0]} file published on {ft_session.portal_base_url}.")
-        pprint(answer)
+        pprint(answer.json())
+    elif answer.status_code == 404:
+        raise MD2FTError(f"Error while publishing <{zipfile['file'][0]}> on <{ft_session.portal_base_url}>, "
+                         f"{answer.json()['message']} (for url={answer.url})")
     else:
-        logger.info(
-            f"DRY RUN: {zipfile['file'][0]} file NOT published."
-        )
+        raise MD2FTError(f"Error while publishing <{zipfile['file'][0]}> on <{ft_session.portal_base_url}>,"
+                         f"source: <{source_id}>, status: <{answer.status_code}>, url: {answer.url})")
 
 
 def _get_meta_values(key: str, metadatas: List[dict], first_value_only: bool):
@@ -546,7 +565,7 @@ def add_options(options):
     "output",
     metavar="<archive_file>",
     type=click.Path(file_okay=True, dir_okay=False, writable=True),
-    help="output path of the FTML archive to be created",
+    help="output path of the FT Archive to be created",
     show_default=True,
     default="ftml_content.zip",
 )
@@ -557,12 +576,12 @@ def collect(
         implicit_meta: bool,
         dev_mode: bool,
 ):
-    """Collect documentation and create an FTML archive based on the metadata found in
+    """Collect documentation and create an FT Archive based on the metadata found in
     the Markdown files (unless implicit_meta is set as True).
     The archive may then be published to a Fluid Topics portal using the `publish` command.
     """
     do_collect(input, output, included_audiences, implicit_meta, dev_mode)
-    logger.info(f"Written FTML archive in: {output}.")
+    logger.info(f"Written FT Archive in: {output}.")
     logger.info(f"You can push the archive to FT portal with:")
     logger.info(f"  md2ft publish {output} <ft_portal_url>")
 
@@ -608,7 +627,7 @@ def publish(
         source_id: str,
         dry_run: bool,
 ):
-    """Upload and publish an FTML archive to an FT portal."""
+    """Upload and publish an FT Archive to an FT portal."""
     session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
     do_publish(zip, session, source_id, dry_run)
 
